program rpncalc;
{$X-}

(* RPN calculator program

   by: Jon Dart
       Department of Anthropology
       UCSD C-001
       La Jolla, CA 92093

   Version 1.3, 18-Nov-84 *)

(* compile with Turbo Pascal *)

const
     ctrl_d    = #4;
     bel       = #7;
     bks       = #8;
     cr	       = #13;
     ctrl_x    = #24;
     space     = #32;
     del       = #127;
     width     = 16; (* field width for numeric output *)
     stack_depth    = 4;

type
     stacktype = array[0..stack_depth] of real;
     numstr = string[12];

var
     cmds,numbers   :set of char;
     decimals	    :integer;  (* # decimal places to display *)
     i,numlen	    :integer;
     ch		    :char;
     numbuf	    :numstr;
     endofline,exit :boolean;
     stack	    :stacktype;

function keyin:char;
(* returns next key pressed on keyboard *)
(* the key is not echoed *)
var    key :integer;
begin
     repeat
	key := bdos(6,255);
     until key<>0;
     keyin := chr(key);
end;

procedure echo(ch:char);
(* echo chars, print control chars. as "^A", etc. *)
begin
    if ord(ch)>=32 then
	write(ch)
    else
	write('^',chr(ord(ch)+ord('A')-1));
end;

function uc(ch:char):char;
(* makes character upper-case *)

begin
     if ch in ['a'..'z'] then uc:=chr(ord(ch)-ord('a')+ord('A'))
     else uc := ch;
end;

procedure give_instructions;
begin
     writeln('Commands:');
     writeln;
     writeln('+ = Add');
     writeln('- = Subtract');
     writeln('* = Multiply');
     writeln('/ = Divide');
     writeln('^ = Exponent');
     writeln('% = Percent');
     writeln('D = Display Stack');
     writeln('R = Square Root');
     writeln('W = Set Number of Decimals');
     writeln('X = Exchange Top Two Levels of Stack');
     writeln('^D = Exit to CP/M');
     writeln('^X = Clear Entry');
     writeln;
end;

procedure docommand(ch:char;var numbuf:numstr;var exit:boolean);
(* do the command corresponding to character ch *)

var  temp,
     t,
     value	    :real;
     i,code	    :integer;
     decstr	    :numstr;
     error	    :boolean;

procedure pop; (* pop stack *)
var i :integer;
begin
     for i:=0 to stack_depth-1 do
	  stack[i]:=stack[i+1];
end;

procedure push(value:real);
(* push value onto stack *)

var  i	  :integer;
begin
     for i:=stack_depth downto 2 do
	  stack[i] := stack[i-1];
     stack[1] := value;
end (*push*);

procedure numout(num:real);
(* output num according to current format *)
begin
     writeln(num:width:decimals);
end;

begin (* docommand *)
     error := false;
     if (length(numbuf) > 0) then
     begin (* decode number in numbuf *)
	  val(numbuf,t,code);
	  if code <> 0 then
	  begin
	       error := true;
	       writeln('Illegal number')
	  end
	  else push(t);
     end
     else if ch=cr then push(stack[1]);
     if not error then
     case ch of
       '?': give_instructions;
       '+': begin
		stack[2] := stack[2] + stack[1];
		pop;
		numout(stack[1]);
	    end;
       '-': begin
		stack[2] := stack[2] - stack[1];
		pop;
		numout(stack[1]);
	    end;
       '*': begin
		stack[2] := stack[2] * stack[1];
		pop;
		numout(stack[1]);
	    end;
       '/': begin
		stack[2] := stack[2] / stack[1];
		pop;
		numout(stack[1]);
	    end;
       '^': begin
		stack[2] := exp(stack[1]*ln(stack[2]));
		pop;
		numout(stack[1]);
	    end;
       '%': begin
		stack[2] := (stack[1]/100)*stack[2];
		pop;
		numout(stack[1]);
	    end;
       'X': begin
		temp:=stack[2]; stack[2]:=stack[1]; stack[1]:=temp;
		numout(stack[1]); end;
       'D': begin
		for i:=1 to stack_depth do
		    numout(stack[i]);
	    end;
       'R': begin
		stack[1]:=sqrt(stack[1]); numout(stack[1]);
	    end;
       'W': begin
	       write('Number of Decimals? ');
	       readln(decstr);
	       if length(decstr)>0 then
	       begin
		   val(decstr,value,code);
		   if code=0 then
		   begin
			decimals := trunc(value);
			numout(stack[1]);
		   end
		   else
			Writeln('Illegal number.');
	       end;
	    end;
	cr: begin
		numout(stack[1]);
	    end;
    ctrl_d: exit:=true;
    ctrl_x: (* do nothing *);
     end (*case*)
end (* docommand *);

begin (* main program *)

     lowvideo;
     writeln;
     writeln('RPN Calculator');
     writeln('by Jon Dart ... Version 1.3 (18-Nov-84)');
     give_instructions;
     cmds      := ['+','-','*','/','%','^','X','D','R','W','?',ctrl_d,ctrl_x,
		   cr];
     numbers   := ['0'..'9','.'];
     exit      := false;
     decimals  := 2;
     for i:=1 to stack_depth do stack[i]:=0;
     repeat
	  write('#');
	  numbuf := ''; numlen := 0; endofline := false;
	  repeat
	       ch := uc(keyin);
	       if ch in cmds then (* have command *)
	       begin
		    if ch <> cr then echo(ch);
		    writeln;
		    docommand(ch,numbuf,exit);
		    endofline := true; (* treat command as end of line *)
	       end
	       else if ch in numbers then
	       begin
		    write(ch);
		    numbuf := numbuf + ch;
		    numlen := numlen + 1;
	       end
	       else if ch in [bks,del] then
	       begin
		    if numlen > 0 then
		    begin
			write(bks,space,bks);
			delete(numbuf,numlen,1);
			numlen := numlen - 1;
		    end
	       end
	       else
		    write(bel); (* error, ding bell *)
	  until endofline;
     until exit;
end.
